'use strict';


/* dependencies */
const _ = require('lodash');
const validator = require('validator');
const { getString } = require('@lykmapipo/env');
const { parsePhoneNumber } = require('@lykmapipo/phone');
const mongoose = require('mongoose');
const MongooseError = mongoose.Error;
const SchemaString = mongoose.Schema.Types.String;


/* custom validations error message */
MongooseError.messages.String.email =
  ('`{VALUE}` is not a valid email address for path `{PATH}`.');
MongooseError.messages.String.macaddress =
  ('`{VALUE}` is not a valid mac address for path `{PATH}`.');
MongooseError.messages.String.ip =
  ('`{VALUE}` is not a valid ip address for path `{PATH}`.');
MongooseError.messages.String.fqdn =
  ('`{VALUE}` is not a valid fully qualified domain name for path `{PATH}`.');
MongooseError.messages.String.alpha =
  ('`{VALUE}` is not a valid alpha value for path `{PATH}`.');
MongooseError.messages.String.alphanumeric =
  ('`{VALUE}` is not a valid alphanumeric value for path `{PATH}`.');
MongooseError.messages.String.md5 =
  ('`{VALUE}` is not a valid md5 value for path `{PATH}`.');
MongooseError.messages.String.creditcard =
  ('`{VALUE}` is not a valid credit card value for path `{PATH}`.');
MongooseError.messages.String.base64 =
  ('`{VALUE}` is not a valid base64 value for path `{PATH}`.');
MongooseError.messages.String.datauri =
  ('`{VALUE}` is not a valid Data URI value for path `{PATH}`.');
MongooseError.messages.String.mimetype =
  ('`{VALUE}` is not a valid mimetype value for path `{PATH}`.');
MongooseError.messages.String.url =
  ('`{VALUE}` is not a valid URL value for path `{PATH}`.');
MongooseError.messages.String.phone =
  ('`{VALUE}` is not a valid phone number value for path `{PATH}`.');
MongooseError.messages.String.mobile =
  ('`{VALUE}` is not a valid mobile phone number value for path `{PATH}`.');
MongooseError.messages.String.fixedline =
  ('`{VALUE}` is not a valid fixedline phone number value for path `{PATH}`.');
MongooseError.messages.String.jwt =
  ('`{VALUE}` is not a valid json web token for path `{PATH}`.');
MongooseError.messages.String.hexadecimal =
  ('`{VALUE}` is not a valid hexadecimal value for path `{PATH}`.');
MongooseError.messages.String.hexacolor =
  ('`{VALUE}` is not a valid hexacolor value for path `{PATH}`.');


/**
 * Sets email validator.
 *
 * ####Example:
 *
 *     var s = new Schema({ n: { type: String, email: true })
 *     var M = db.model('M', s)
 *     var m = new M({ n: 'a@b' })
 *     m.save(function (err) {
 *       console.error(err) // validator error
 *       m.n = 'a@b.com';
 *       m.save() // success
 *     })
 *
 * @param {Boolean|Object} options email validation options
 * @param {String} [message] optional custom error message
 * @return {SchemaType} this
 * @see Customized Error Messages #error_messages_MongooseError-messages
 * @api public
 */
SchemaString.prototype.email = function email(options, message) {
  if (this.emailValidator) {
    this.validators = this.validators.filter(function (v) {
      return v.validator !== this.emailValidator;
    }, this);
  }

  if (options !== null && options !== undefined) {
    const msg =
      (options.message || message || MongooseError.messages.String.email);
    this.validators.push({
      validator: this.emailValidator = function (v) {
        return validator.isEmail(String(v));
      },
      message: msg,
      type: 'email'
    });
  }

  return this;

};


/**
 * Sets macaddress validator.
 *
 * ####Example:
 *
 *     var s = new Schema({ n: { type: String, macaddress: true })
 *     var M = db.model('M', s)
 *     var m = new M({ n: 'abc' })
 *     m.save(function (err) {
 *       console.error(err) // validator error
 *       m.n = 'FF:FF:FF:FF:FF:FF';
 *       m.save() // success
 *     })
 *
 * @param {Boolean|Object} options macaddress validation options
 * @param {String} [message] optional custom error message
 * @return {SchemaType} this
 * @see Customized Error Messages #error_messages_MongooseError-messages
 * @api public
 */
SchemaString.prototype.macaddress = function macaddress(options, message) {

  if (this.macaddressValidator) {
    this.validators = this.validators.filter(function (v) {
      return v.validator !== this.macaddressValidator;
    }, this);
  }

  if (options !== null && options !== undefined) {
    const msg =
      (options.message || message || MongooseError.messages.String.macaddress);
    this.validators.push({
      validator: this.macaddressValidator = function (v) {
        return validator.isMACAddress(String(v));
      },
      message: msg,
      type: 'macaddress'
    });
  }

  return this;

};


/**
 * Sets ip validator.
 *
 * ####Example:
 *
 *     var s = new Schema({ n: { type: String, ip: true })
 *     var M = db.model('M', s)
 *     var m = new M({ n: 'abc' })
 *     m.save(function (err) {
 *       console.error(err) // validator error
 *       m.n = '127.0.0.1';
 *       m.save() // success
 *     })
 *
 * @param {Boolean|Object} options ip validation options
 * @param {String} [message] optional custom error message
 * @return {SchemaType} this
 * @see Customized Error Messages #error_messages_MongooseError-messages
 * @api public
 */
SchemaString.prototype.ip = function ip(options, message) {

  if (this.ipValidator) {
    this.validators = this.validators.filter(function (v) {
      return v.validator !== this.ipValidator;
    }, this);
  }

  if (options !== null && options !== undefined) {
    const msg =
      (options.message || message || MongooseError.messages.String.ip);
    this.validators.push({
      validator: this.ipValidator = function (v) {
        return validator.isIP(String(v));
      },
      message: msg,
      type: 'ip'
    });
  }

  return this;

};


/**
 * Sets fqdn validator.
 *
 * ####Example:
 *
 *     var s = new Schema({ n: { type: String, fqdn: true })
 *     var M = db.model('M', s)
 *     var m = new M({ n: 'abc' })
 *     m.save(function (err) {
 *       console.error(err) // validator error
 *       m.n = 'domain.com';
 *       m.save() // success
 *     })
 *
 * @param {Boolean|Object} options fqdn validation options
 * @param {String} [message] optional custom error message
 * @return {SchemaType} this
 * @see Customized Error Messages #error_messages_MongooseError-messages
 * @api public
 */
SchemaString.prototype.fqdn = function fqdn(options, message) {

  if (this.fqdnValidator) {
    this.validators = this.validators.filter(function (v) {
      return v.validator !== this.fqdnValidator;
    }, this);
  }

  if (options !== null && options !== undefined) {
    const msg =
      (options.message || message || MongooseError.messages.String.fqdn);
    this.validators.push({
      validator: this.fqdnValidator = function (v) {
        return validator.isFQDN(String(v));
      },
      message: msg,
      type: 'fqdn'
    });
  }

  return this;

};


/**
 * Sets alpha validator.
 *
 * ####Example:
 *
 *     var s = new Schema({ n: { type: String, alpha: true })
 *     var M = db.model('M', s)
 *     var m = new M({ n: 'abc1' })
 *     m.save(function (err) {
 *       console.error(err) // validator error
 *       m.n = 'abc';
 *       m.save() // success
 *     })
 *
 * @param {Boolean|Object} options alpha validation options
 * @param {String} [message] optional custom error message
 * @return {SchemaType} this
 * @see Customized Error Messages #error_messages_MongooseError-messages
 * @api public
 */
SchemaString.prototype.alpha = function alpha(options, message) {

  if (this.alphaValidator) {
    this.validators = this.validators.filter(function (v) {
      return v.validator !== this.alphaValidator;
    }, this);
  }

  if (options !== null && options !== undefined) {
    const msg =
      (options.message || message || MongooseError.messages.String.alpha);
    this.validators.push({
      validator: this.alphaValidator = function (v) {
        return validator.isAlpha(String(v));
      },
      message: msg,
      type: 'alpha'
    });
  }

  return this;

};


/**
 * Sets alphanumeric validator.
 *
 * ####Example:
 *
 *     var s = new Schema({ n: { type: String, alphanumeric: true })
 *     var M = db.model('M', s)
 *     var m = new M({ n: 'abc' })
 *     m.save(function (err) {
 *       console.error(err) // validator error
 *       m.n = 'abc1';
 *       m.save() // success
 *     })
 *
 * @param {Boolean|Object} options alphanumeric validation options
 * @param {String} [message] optional custom error message
 * @return {SchemaType} this
 * @see Customized Error Messages #error_messages_MongooseError-messages
 * @api public
 */
SchemaString.prototype.alphanumeric = function alphanumeric(options, message) {

  if (this.alphanumericValidator) {
    this.validators = this.validators.filter(function (v) {
      return v.validator !== this.alphanumericValidator;
    }, this);
  }

  if (options !== null && options !== undefined) {
    const msg =
      (options.message || message || MongooseError.messages.String.alphanumeric);
    this.validators.push({
      validator: this.alphanumericValidator = function (v) {
        return validator.isAlphanumeric(String(v));
      },
      message: msg,
      type: 'alphanumeric'
    });
  }

  return this;

};


/**
 * Sets md5 validator.
 *
 * ####Example:
 *
 *     var s = new Schema({ n: { type: String, md5: true })
 *     var M = db.model('M', s)
 *     var m = new M({ n: 'abc' })
 *     m.save(function (err) {
 *       console.error(err) // validator error
 *       m.n = 'd94f3f016ae679c3008de268209132f2';
 *       m.save() // success
 *     })
 *
 * @param {Boolean|Object} options md5 validation options
 * @param {String} [message] optional custom error message
 * @return {SchemaType} this
 * @see Customized Error Messages #error_messages_MongooseError-messages
 * @api public
 */
SchemaString.prototype.md5 = function md5(options, message) {

  if (this.md5Validator) {
    this.validators = this.validators.filter(function (v) {
      return v.validator !== this.md5Validator;
    }, this);
  }

  if (options !== null && options !== undefined) {
    const msg =
      (options.message || message || MongooseError.messages.String.md5);
    this.validators.push({
      validator: this.md5Validator = function (v) {
        return validator.isMD5(String(v));
      },
      message: msg,
      type: 'md5'
    });
  }

  return this;

};


/**
 * Sets uuid validator.
 *
 * ####Example:
 *
 *     var s = new Schema({ n: { type: String, uuid: true })
 *     var M = db.model('M', s)
 *     var m = new M({ n: 'abc' })
 *     m.save(function (err) {
 *       console.error(err) // validator error
 *       m.n = 'A987FBC9-4BED-5078-AF07-9141BA07C9F3';
 *       m.save() // success
 *     })
 *
 * @param {Boolean|Object} options uuid validation options
 * @param {String} [message] optional custom error message
 * @return {SchemaType} this
 * @see Customized Error Messages #error_messages_MongooseError-messages
 * @api public
 */
SchemaString.prototype.uuid = function uuid(options, message) {

  if (this.uuidValidator) {
    this.validators = this.validators.filter(function (v) {
      return v.validator !== this.uuidValidator;
    }, this);
  }

  if (options !== null && options !== undefined) {
    const msg =
      (options.message || message || MongooseError.messages.String.uuid);
    this.validators.push({
      validator: this.uuidValidator = function (v) {
        return validator.isUUID(String(v));
      },
      message: msg,
      type: 'uuid'
    });
  }

  return this;

};


/**
 * Sets creditcard validator.
 *
 * ####Example:
 *
 *     var s = new Schema({ n: { type: String, creditcard: true })
 *     var M = db.model('M', s)
 *     var m = new M({ n: 'abc' })
 *     m.save(function (err) {
 *       console.error(err) // validator error
 *       m.n = '4716-2210-5188-5662';
 *       m.save() // success
 *     })
 *
 * @param {Boolean|Object} options creditcard validation options
 * @param {String} [message] optional custom error message
 * @return {SchemaType} this
 * @see Customized Error Messages #error_messages_MongooseError-messages
 * @api public
 */
SchemaString.prototype.creditcard = function creditcard(options, message) {

  if (this.creditcardValidator) {
    this.validators = this.validators.filter(function (v) {
      return v.validator !== this.creditcardValidator;
    }, this);
  }

  if (options !== null && options !== undefined) {
    const msg =
      (options.message || message || MongooseError.messages.String.creditcard);
    this.validators.push({
      validator: this.creditcardValidator = function (v) {
        return validator.isCreditCard(String(v));
      },
      message: msg,
      type: 'creditcard'
    });
  }

  return this;

};


/**
 * Sets base64 validator.
 *
 * ####Example:
 *
 *     var s = new Schema({ n: { type: String, base64: true })
 *     var M = db.model('M', s)
 *     var m = new M({ n: 'abc' })
 *     m.save(function (err) {
 *       console.error(err) // validator error
 *       m.n = 'Zm9vYmE=';
 *       m.save() // success
 *     })
 *
 * @param {Boolean|Object} options base64 validation options
 * @param {String} [message] optional custom error message
 * @return {SchemaType} this
 * @see Customized Error Messages #error_messages_MongooseError-messages
 * @api public
 */
SchemaString.prototype.base64 = function base64(options, message) {

  if (this.base64Validator) {
    this.validators = this.validators.filter(function (v) {
      return v.validator !== this.base64Validator;
    }, this);
  }

  if (options !== null && options !== undefined) {
    const msg =
      (options.message || message || MongooseError.messages.String.base64);
    this.validators.push({
      validator: this.base64Validator = function (v) {
        return validator.isBase64(String(v));
      },
      message: msg,
      type: 'base64'
    });
  }

  return this;

};


/**
 * Sets data uri validator.
 *
 * ####Example:
 *
 *     var s = new Schema({ n: { type: String, datauri: true })
 *     var M = db.model('M', s)
 *     var m = new M({ n: 'abc' })
 *     m.save(function (err) {
 *       console.error(err) // validator error
 *       m.n = 'Zm9vYmE=';
 *       m.save() // success
 *     })
 *
 * @param {Boolean|Object} options datauri validation options
 * @param {String} [message] optional custom error message
 * @return {SchemaType} this
 * @see Customized Error Messages #error_messages_MongooseError-messages
 * @api public
 */
SchemaString.prototype.datauri = function datauri(options, message) {

  if (this.datauriValidator) {
    this.validators = this.validators.filter(function (v) {
      return v.validator !== this.datauriValidator;
    }, this);
  }

  if (options !== null && options !== undefined) {
    const msg =
      (options.message || message || MongooseError.messages.String.datauri);
    this.validators.push({
      validator: this.dataUriValidator = function (v) {
        return validator.isDataURI(String(v));
      },
      message: msg,
      type: 'datauri'
    });
  }

  return this;

};


/**
 * Sets mimetype validator.
 *
 * ####Example:
 *
 *     var s = new Schema({ n: { type: String, mimetype: true })
 *     var M = db.model('M', s)
 *     var m = new M({ n: 'abc' })
 *     m.save(function (err) {
 *       console.error(err) // validator error
 *       m.n = 'application/json';
 *       m.save() // success
 *     })
 *
 * @param {Boolean|Object} options mimetype validation options
 * @param {String} [message] optional custom error message
 * @return {SchemaType} this
 * @see Customized Error Messages #error_messages_MongooseError-messages
 * @api public
 */
SchemaString.prototype.mimetype = function mimetype(options, message) {

  if (this.mimetypeValidator) {
    this.validators = this.validators.filter(function (v) {
      return v.validator !== this.mimetypeValidator;
    }, this);
  }

  if (options !== null && options !== undefined) {
    const msg =
      (options.message || message || MongooseError.messages.String.mimetype);
    this.validators.push({
      validator: this.mimetypeValidator = function (v) {
        return validator.isMimeType(String(v));
      },
      message: msg,
      type: 'mimetype'
    });
  }

  return this;

};


/**
 * Sets url validator.
 *
 * ####Example:
 *
 *     var s = new Schema({ n: { type: String, url: true })
 *     var M = db.model('M', s)
 *     var m = new M({ n: 'abc' })
 *     m.save(function (err) {
 *       console.error(err) // validator error
 *       m.n = 'https://www.npmjs.com';
 *       m.save() // success
 *     })
 *
 * @param {Boolean|Object} options url validation options
 * @param {String} [message] optional custom error message
 * @return {SchemaType} this
 * @see Customized Error Messages #error_messages_MongooseError-messages
 * @api public
 */
SchemaString.prototype.url = function url(options, message) {

  if (this.urlValidator) {
    this.validators = this.validators.filter(function (v) {
      return v.validator !== this.urlValidator;
    }, this);
  }

  if (options !== null && options !== undefined) {
    const msg =
      (options.message || message || MongooseError.messages.String.url);
    this.validators.push({
      validator: this.urlValidator = function (v) {
        return validator.isURL(String(v));
      },
      message: msg,
      type: 'url'
    });
  }

  return this;

};


/**
 * Converts the first character of string to upper case and the remaining to
 * lower case.
 *
 * ####Example:
 *
 *     var s = new Schema({ name: { type: String, capitalize: true }})
 *     var M = db.model('M', s);
 *     var m = new M({ name: 'fred'});
 *     console.log(m.name) // 'Fred'
 *
 * @api public
 * @return {SchemaType} this
 */

SchemaString.prototype.capitalize = function capitalize(shouldApply) {
  if (arguments.length > 0 && !shouldApply) {
    return this;
  }
  return this.set(function (v /*, self*/ ) {
    const value = _.capitalize(v);
    return value;
  });
};


/**
 * Converts string to start case(or title case).
 *
 * ####Example:
 *
 *     var s = new Schema({ name: { type: String, startcase: true }})
 *     var M = db.model('M', s);
 *     var m = new M({ name: 'fred fuga'});
 *     console.log(m.name) // 'Fred Fuga'
 *
 * @api public
 * @return {SchemaType} this
 */

SchemaString.prototype.startcase = function startcase(shouldApply) {
  if (arguments.length > 0 && !shouldApply) {
    return this;
  }
  return this.set(function (v /*, self*/ ) {
    const value = _.startCase(_.toLower(v));
    return value;
  });
};


/**
 * Sets phone validator.
 *
 * ####Example:
 *
 *     var s = new Schema({ n: { type: String, phone: true })
 *     var M = db.model('M', s)
 *     var m = new M({ n: 'abc' })
 *     m.save(function (err) {
 *       console.error(err) // validator error
 *       m.n = '255714676767';
 *       m.save() // success
 *     })
 *
 * @param {Boolean|Object} options phone validation options
 * @param {String} [message] optional custom error message
 * @return {SchemaType} this
 * @see Customized Error Messages #error_messages_MongooseError-messages
 * @api public
 */
SchemaString.prototype.phone = function phone(options, message) {

  if (this.phoneValidator) {
    this.validators = this.validators.filter(function (v) {
      return v.validator !== this.phoneValidator;
    }, this);
  }

  // obtain path name
  const pathName = this.path;

  if (options !== null && options !== undefined) {
    // collect validator options
    const { mobile, fixedline, e164 } = options;
    const countries =
      _.compact(([getString('DEFAULT_COUNTRY_CODE')].concat(options.countries)));
    let msg = (message || MongooseError.messages.String.phone);
    msg = (options.message || msg);
    msg = (mobile ? MongooseError.messages.String.mobile : msg);
    msg = (fixedline ? MongooseError.messages.String.fixedline : msg);
    // collect validator
    this.validators.push({
      validator: this.phoneValidator = function (v) {
        const phoneNumber = parsePhoneNumber(String(v), ...countries);
        const { isMobile, isFixedLine, isTollFree } = phoneNumber || {};
        let isValid = (phoneNumber ? phoneNumber.isValid : false);
        isValid = (mobile ? isValid && isMobile : isValid);
        isValid =
          (fixedline ? isValid && (isFixedLine || isTollFree) : isValid);
        // transform
        if (this.constructor.name === 'model') {
          if (isValid && e164) {
            this[pathName] = phoneNumber.e164NoPlus;
          }
        }
        // return validation status
        return isValid;
      },
      message: msg,
      type: 'phone'
    });
  }

  return this;

};


/**
 * Sets jwt validator.
 *
 * ####Example:
 *
 *     var s = new Schema({ n: { type: String, jwt: true })
 *     var M = db.model('M', s)
 *     var m = new M({ n: 'a@b' })
 *     m.save(function (err) {
 *       console.error(err) // validator error
 *       m.n = 'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.
 *       eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiaWF0IjoxNTE2Mj
 *       M5MDIyfQ.SflKxwRJSMeKKF2QT4fwpMeJf36POk6yJV_adQssw5c';
 *       m.save() // success
 *     })
 *
 * @param {Boolean|Object} options jwt validation options
 * @param {String} [message] optional custom error message
 * @return {SchemaType} this
 * @see Customized Error Messages #error_messages_MongooseError-messages
 * @api public
 */
SchemaString.prototype.jwt = function jwt(options, message) {
  if (this.jwtValidator) {
    this.validators = this.validators.filter(function (v) {
      return v.validator !== this.jwtValidator;
    }, this);
  }

  if (options !== null && options !== undefined) {
    const msg =
      (options.message || message || MongooseError.messages.String.jwt);
    this.validators.push({
      validator: this.jwtValidator = function (v) {
        return validator.isJWT(String(v));
      },
      message: msg,
      type: 'jwt'
    });
  }

  return this;

};

/**
 * Sets hexadecimal validator.
 *
 * ####Example:
 *
 *     var s = new Schema({ n: { type: String, hexadecimal: true })
 *     var M = db.model('M', s)
 *     var m = new M({ n: 'a@b' })
 *     m.save(function (err) {
 *       console.error(err) // validator error
 *       m.n = 'FF0044';
 *       m.save() // success
 *     })
 *
 * @param {Boolean|Object} options hexadecimal validation options
 * @param {String} [message] optional custom error message
 * @return {SchemaType} this
 * @see Customized Error Messages #error_messages_MongooseError-messages
 * @api public
 */
SchemaString.prototype.hexadecimal = function hexadecimal(options, message) {
  if (this.hexadecimalValidator) {
    this.validators = this.validators.filter(function (v) {
      return v.validator !== this.hexadecimalValidator;
    }, this);
  }

  if (options !== null && options !== undefined) {
    const msg =
      (options.message || message || MongooseError.messages.String.hexadecimal);
    this.validators.push({
      validator: this.hexadecimalValidator = function (v) {
        return validator.isHexadecimal(String(v));
      },
      message: msg,
      type: 'hexadecimal'
    });
  }

  return this;

};

/**
 * Sets hexacolor validator.
 *
 * ####Example:
 *
 *     var s = new Schema({ n: { type: String, hexacolor: true })
 *     var M = db.model('M', s)
 *     var m = new M({ n: 'a@b' })
 *     m.save(function (err) {
 *       console.error(err) // validator error
 *       m.n = '#FF0044';
 *       m.save() // success
 *     })
 *
 * @param {Boolean|Object} options hexacolor validation options
 * @param {String} [message] optional custom error message
 * @return {SchemaType} this
 * @see Customized Error Messages #error_messages_MongooseError-messages
 * @api public
 */
SchemaString.prototype.hexacolor = function hexacolor(options, message) {
  if (this.hexacolorValidator) {
    this.validators = this.validators.filter(function (v) {
      return v.validator !== this.hexacolorValidator;
    }, this);
  }

  if (options !== null && options !== undefined) {
    const msg =
      (options.message || message || MongooseError.messages.String.hexacolor);
    this.validators.push({
      validator: this.hexacolorValidator = function (v) {
        return validator.isHexColor(String(v));
      },
      message: msg,
      type: 'hexacolor'
    });
  }

  return this;

};
